/**
 * Copyright (c) Vispy Development Team
 * Distributed under the (new) BSD License. See LICENSE.txt for more info.
 *
 * This is a convencience include file with includes some math helper
 * functions.
 */

#include "math/signed-line-distance.glsl"
#include "math/point-to-line-distance.glsl"
#include "math/signed-segment-distance.glsl"
#include "math/circle-through-2-points.glsl"
#include "math/point-to-line-projection.glsl"
